with Spacecraft.Ctors; use Spacecraft.Ctors;
use Spacecraft;

package Known_Satellites with
   SPARK_Mode,
   Abstract_State => State,
   Initializes    => State
is

   Max_Satellites : constant := 10; -- arbitrary

   type Satellite_Reference is limited private;

   --  Insert the satellite Value into the repository of known satellites.
   --  The output Reference provides a reference to that satellite within the
   --  repository and can be used as such in other routines to retrieve and/or
   --  alter the corresponding satellite value.
   procedure Insert_Satellite
     (Value     : in Satellite;
      Reference : out Satellite_Reference)
   with
      Global  => (In_Out => State),
      Depends => (Reference => State,
                  State =>+ Value),
      Pre     => not Full,
      Post    => Valid (Reference);

   --  Update the satellite designated in the repository by Reference with the
   --  new Value.
   procedure Update_Satellite
     (Value     : in Satellite;
      Reference : in Satellite_Reference)
   with
      Global  => (In_Out => State),
      Depends => (State =>+ (Value, Reference)),
      Pre     => Valid (Reference);

   --  Get a copy of the Value of the satellite designated by Reference. The
   --  repository is not changed.
   procedure Get_Satellite
     (Reference : in Satellite_Reference;
      Value     : out Satellite)
   with
      Global  => (Input => State),
      Depends => (Value => (State, Reference)),
      Pre     => Valid (Reference);

   --  A convenience routine that returns a copy of the satellite value
   --  designated by Reference. The repository is not changed.
   function Dereferenced (Reference : Satellite_Reference) return Satellite
   with
      Global  => (Input => State),
      Depends => (Dereferenced'Result => (State, Reference)),
      Pre     => Valid (Reference);

   function Full return Boolean
   with
      Global  => (Input => State),
      Depends => (Full'Result => State);

   function Valid (Reference : Satellite_Reference) return Boolean
   with
      Global  => null,
      Depends => (Valid'Result => Reference);

private

   type Satellite_Reference is range 0 .. Max_Satellites with
     Default_Value => 0;

   Null_Reference : constant Satellite_Reference := 0;

   subtype Index is Satellite_Reference
      range Satellite_Reference'First + 1 .. Satellite_Reference'Last;

   All_Satellites : array (Index) of Satellite := (others => Null_Satellite)
      with Part_Of => State;

   Satellite_Count : Satellite_Reference := Null_Reference
      with Part_Of => State;

   function Full return Boolean is
      (Satellite_Count = Max_Satellites);

   function Valid (Reference : Satellite_Reference) return Boolean is
      (Reference /= Null_Reference);

end Known_Satellites;
